%Decay tests
clearvars except first last
clc 
close all
lambda=15;
ro=1.025;

filename=[...
'C:\Users\Anna\Dropbox\M14\Modelltest\Res\Data\0001c.bin';
'C:\Users\Anna\Dropbox\M14\Modelltest\Res\Data\0002b.bin'
'C:\Users\Anna\Dropbox\M14\Modelltest\Res\Data\0003b.bin'
'C:\Users\Anna\Dropbox\M14\Modelltest\Res\Data\0004b.bin'
'C:\Users\Anna\Dropbox\M14\Modelltest\Res\Data\0005a.bin'
'C:\Users\Anna\Dropbox\M14\Modelltest\Res\Data\0006d.bin'

];

V_dis=38.170; %m^3
GM_T=1.62; %m
GM_L=1.62; %m
Mass=38745; %kg

C33=ro*1000*9.81*3*pi*0.75^2;
C44=ro*1000*9.81*V_dis*GM_T;
C55=ro*1000*9.81*V_dis*GM_T;


for i=1:size(filename,1)

clc
clear locs pks ksi amp

disp(' ') 
disp('Filename: ')
disp(filename(i,:))
disp(' ')

%Read results from .bin file
[a1,a2,Time,Frame,Fx_58,Fy_58,Fz_58,Fx_96,Fy_96,Fz_96,Xpos,Ypos,Zpos,Roll,Pitch,Yaw,Wave]=catman_read(filename(i,:));

% Scaling
Time=Time*sqrt(lambda);
Xpos=lambda*Xpos;
Ypos=lambda*Ypos;
Zpos=lambda*Zpos;


%Plot for visual inspection
fig = figure;
set(fig,'units','normalized','outerposition',[0.5 0 0.5 1]);
subplot(3,1,1); plot(Time, Zpos); title('Zpos'); grid on
subplot(3,1,2); plot(Time, Roll); title('Roll'); grid on
subplot(3,1,3); plot(Time, Pitch); title('Pitch'); grid on

%Find scaled sampling frequency
fi=Time(3)-Time(2);
fc=1/fi; 




% Set mode (heave/pitch/roll)
if i==1 || i==4
    mode=33;
elseif i==2 || i==5
    mode=55;
elseif i==3 || i==6
    mode=44;
end



finish=0;
while finish==0
    

% Extracting best time periods 
disp('Enter start and end time, enter 0 for both for reset.')
disp('For decay test, skip 1st oscillation to avoid transient effects.')
disp('Mode to consider:')
disp(mode)
% 
 first(i)=round(fc*input('Enter start time: '));
 last(i)=round(fc*input('Enter end time: '));


if first(i)==0; first(i)=1; end
if last(i)==0; last(i)=length(Time); end


%Extracting desired time periods    
Time_red=Time(1,first(i):last(i));
Zpos_red=Zpos(1,first(i):last(i));
Roll_red=Roll(1,first(i):last(i));
Pitch_red=Pitch(1,first(i):last(i));


%New plots

subplot(3,1,1); plot(Time_red, Zpos_red); title('Zpos'); grid on
subplot(3,1,2); plot(Time_red, Roll_red); title('Roll'); grid on
subplot(3,1,3); plot(Time_red, Pitch_red); title('Pitch');grid on 


finish=input('All good? yes=1, no=0 ');
end

%Remove bias
Zpos_red=Zpos_red-(mean(Zpos_red));
Pitch_red=Pitch_red-mean(Pitch_red);
Roll_red=Roll_red-mean(Roll_red);

%Save plot to file 
 h=figure;
 if i==1 || i==4; mode=33;
 plot(Time_red, Zpos_red); title('Heave'); xlabel('Time (s)'); ylabel('Z-position (m)')
    
 elseif i==2 || i==5; mode=55;
 plot(Time_red, Pitch_red); title('Pitch'); xlabel('Time (s)'); ylabel('Pitch-angle (deg)')

 elseif i==3 || i==6; mode=44;
 plot(Time_red, Roll_red); title('Roll'); xlabel('Time (s)'); ylabel('Roll-angle (deg)')
 end

name = sprintf('Decayplots/%s', filename(i,47:51));
print(h,'-djpeg',name);
close all
disp('Plot saved to file.')
disp(name)
disp(' ')


% Enter a rough estimate for td:
wavet=round(6*fc);

% Find peaks and their locations 
% Set stiffness according to defined mode
if mode==33
[pks locs]=findpeaks(Zpos_red,'minpeakdistance',wavet); C=C33;
elseif mode==44
[pks locs]=findpeaks(Roll_red,'minpeakdistance',wavet); C=C44;
elseif mode==55
[pks locs]=findpeaks(Pitch_red,'minpeakdistance',wavet); C=C55;
end

close all 

% Estimate damping 
for j=1:length(pks)-1
logdec(j)=log(pks(j)/pks(j+1)); %Logarithmic decrement
ksi(j)=logdec(j)/(2*3.14);      %Damping ratio
amp(j)=(pks(j)+pks(j+1))/2;  %Mean amplitude 
end

% Eigenperiods and eigenfrequencies
Td=Time_red(locs(2))-Time_red(locs(1)); % Damped eigenperiod
T0=Td/sqrt(1-ksi(1));%Undamped eigenperiod
omega_0=2*pi/T0;


%Added mass from Eigen frequency:
Mass_total=p_eq/(2*omega_0*ksi); 
Mass_added=Mass_total-Mass;

% Equivalent damping
p_eq=2*C*ksi/omega_0;

%Plot p_eq vs. mean amplitude. 
name=sprintf('Results/polyfit_case%d',i);
figure(1)
[mdl gof]=fit(amp',p_eq', 'poly1' );
plot(mdl)
hold on
plot(amp,p_eq,'+')
xlabel('Mean amplitude (m)')
ylabel('Equivalend damping (Nm/s)')
print(1,'-djpeg',name)

%Print results to file. 
name = sprintf('Results/Result-%s.txt', filename(i,47:51));
fid2=fopen(name,'w');

fprintf(fid2,'%f\n',mdl.p2);                % Linear contribution is the damping at x=0
fprintf(fid2,'%f\n',mdl.p1);                % Quadratic contribution is the gradient
fprintf(fid2,'%f %f %f \n',p_eq(1:3));      % Equivalent damping
fprintf(fid2,'%f\n',Mass_added);            % Added mass
fprintf(fid2,'%f\n',Td);                    % Damped Eigen period
fprintf(fid2,'%f\n',T0);                    % Undamped Eigen period

disp('Results are now written to file:')
disp(name)

end







