%Run
clear all 
filename=['C:\Users\Anna\Dropbox\M14\SIMA\Postprosessering\SIMO\Input\platform_dir00.txt';...
    'C:\Users\Anna\Dropbox\M14\SIMA\Postprosessering\SIMO\Input\platform_dir90.txt'];

delimiter=' ';
r=354;
c=1;

%Input time step:
fi=0.05;

for i=1:size(filename,1)
    
%Read result from file, store to variables
result=dlmread(filename(i,:),delimiter,r,c);


% Res(:,10)=result(:,1); %Wave
% Res(:,1)=result(:,3);  %Xforce
% Res(:,2)=result(:,4);  %Yforce
% Res(:,3)=result(:,5);  %Zforce
% Res(:,4)=result(:,192);   %Xpos
% Res(:,5)=result(:,193);  %Ypos
% Res(:,6)=result(:,194);   %Zpos
% Res(:,7)=result(:,195);  %Xrot
% Res(:,8)=result(:,196);   %Yrot
% Res(:,9)=result(:,197);   %Zrot


% 
% if i==1
%     save('Results/Platform/Resultdir00','Res')
%     elseif i==2
%         save('Results/Platform/Resultdir90','Res')
% end

if i==1;
 load Results/Platform/Resultdir00
elseif i==2
    load Results/Platform/Resultdir90
end


%On forces: a rough check shows that fixel-forces give almost the same as
%"total force". It is therefore reasonable to assume that to use Xforce is
%OK. In any ways; the transducers also measure total force, including wave
%forces, so this must be the correct choice. 


%Channel explanation
fid2=fopen('Results/Platform/Channels.txt','w');
fprintf(fid2,'Channel 1: Xforce');
fprintf(fid2,'Channel 2: Yforce');
fprintf(fid2,'Channel 3: Zforce');
fprintf(fid2,'Channel 4: Xpos');
fprintf(fid2,'Channel 5: Ypos');
fprintf(fid2,'Channel 6: Zpos');
fprintf(fid2,'Channel 7: Xrot');
fprintf(fid2,'Channel 8: Yrot');
fprintf(fid2,'Channel 9: Zrot');


Res_mag=zeros(size(Res));

%Remove bias
for k=1:10
    Res(:,k)=Res(:,k)-mean(Res(:,k));
end

%Find magnitudes
for k=1:10
Res_mag(:,k)=abs(fft(Res(:,k)));
end


%Create frequencies for plotting
num=size(Res,1);
fs=1/fi; 
f=(0:1/(num/2-1):1)*fs/2;
flen=length(f);

T=5;

%Filter
   Wn1=0.4/T; 
   Wn2=5/T;    
    Wp = [Wn1 Wn2]*2/fs; Ws = [Wn1*0.5 Wn2*1.5]*2/fs;  % Normalizing frequency
    Rp = 3; Rs = 4;
    [n,Wn] = buttord(Wp,Ws,Rp,Rs);  % Gives mimimum order of filter   
    [b3,a3] = butter(n,Wn);           % Butterworth filter design  

    Resfil=zeros(size(Res));
for k=1:10
    Resfil(:,k)=filtfilt(b3,a3,Res(:,k));
end

     

%% Find response spectra and transfer functions 
[Wave_spec,fw]=pwelch(Resfil(:,10), [],[],[],fs);


Response_spec=zeros(size(Wave_spec,1),9);
RAO=zeros(size(Wave_spec,1),9);
fr=zeros(size(Response_spec));

for k=1:9
 [Response_spec(:,k),fr(:,k)]=pwelch(Resfil(:,k), [],[],[],fs);
end
 

 

for k=1:9
  RAO(:,k)=sqrt(Response_spec(:,k)./Wave_spec);
end

%%

%Save RAO to struct
RAOstruct(i)=struct('Surge',RAO(:,4),...
                'Sway',RAO(:,5),...
'Heave',RAO(:,6),...
'Roll',RAO(:,7),...
'Pitch',RAO(:,8),...
'Yaw',RAO(:,9),...
'Wave',Wave_spec...
            );


%reduce fr to only one column
fr=fr(:,1);

% Aquire moments for each spectrum
for k=1:9

S=Response_spec(:,k);
dfr=fr(2)-fr(1);
S=S(1:length(fr));

m0=sum(S.*dfr);
m1=sum(fr.*S.*dfr);
m2=sum((fr.^2).*S.*dfr);


%Peak period of response:
[maks ind]=max(Response_spec(:,k));
Tp=1/(f(ind));
Stdev=sqrt(m0);
Sigval=4*sqrt(m0);
T1=m0/m1; %Average period of response
T2=sqrt(m0/m2); %Average zero-crossing period


Mean=mean(Res(:,k));
Max=max(Res(:,k));
Min=min(Res(:,k));
Stdev=std(Res(:,k))


nummax=3*3600/Tp;



%find amplitudes larger than zero.
[amp locs2]=findpeaks(Res(:,k),'minpeakheight',0); % Including all the peaks in the series, not just reduces)
%Expected value of the largest max according to Rayleigh 
exp_max=Stdev*(sqrt(2*log(nummax)+0.5772/sqrt(2*log(nummax))))

%Most probably largest max according to Rayleigh:
prob_max=Stdev*sqrt(2*log(nummax))



%Save Spectral analysis results to file
name=sprintf('Results/Platform/%s_spectral_analysis_Channel_%i.txt', filename(i,69:73),k);
fid3=fopen(name,'w');

fprintf(fid3,'%f\n',m0);
fprintf(fid3,'%f\n',m1);
fprintf(fid3,'%f\n',m2);
fprintf(fid3,'%f\n',Sigval);
fprintf(fid3,'%f\n',T1);
fprintf(fid3,'%f\n',T2);
fprintf(fid3,'%f\n',Tp);
fprintf(fid3,'%f\n',Max);
fprintf(fid3,'%f\n',Min);
fprintf(fid3,'%f\n',Mean);
fprintf(fid3,'%f\n',Stdev);
fprintf(fid3,'%f\n',exp_max);
fprintf(fid3,'%f\n',prob_max);
    
end
end


disp('all good part 1')

%% Part II - plotting of RAOs


T=fr;


%Cut off time series to reduce computing time:
xmin=0.1;
xmax=0.85;
tmin=find(T>xmin);
tmin=tmin(1);
tmax=find(T>xmax);
tmax=tmax(1);


%Colors
cmap=hsv(5);




%% Surge
figure (1);
hold on

for k=1:2
%      
         plot(T,smooth(RAOstruct(k).Surge,10),'Color',cmap(k,:))
         
end

legend('0 deg', '90 deg')

title('Surge transfer functions')
xlabel('Frequency (Hz)')
ylabel('RAO (m/m)')
 xlim([xmin xmax])


print(1,'-djpeg','Results/Platform/RAO_comparison/SurgeRAO');

%%

%Sway
figure(2)


hold on
for k=1:2
    
    plot(T(tmin:tmax),smooth(RAOstruct(k).Sway(tmin:tmax),10),'Color',cmap(k,:))
  
end
legend('0 deg', '90 deg')
xlim([0 0.2]) 
title('Sway transfer functions')
xlabel('Frequency (Hz)')
ylabel('RAO (m/m)')
 xlim([xmin xmax])
% ylim([0 0.5])

print(2,'-djpeg','Results/Platform/RAO_comparison/SwayRAO');

%% Heave


figure(3)
for k=1:2
plot(T(tmin:tmax),smooth(RAOstruct(k).Heave(tmin:tmax),10),'Color',cmap(k,:))
hold on 

end
legend('0 deg', '90 deg')

title('Heave transfer functions')
xlabel('Frequency (Hz)')
ylabel('RAO (m/m)')
 xlim([xmin xmax])
% ylim([0 10])

%Save plot to file. 
print(3,'-djpeg','Results/Platform/RAO_comparison/HeaveRAO');
%%

%Roll
figure(4)
hold on
for k=1:2
    
  plot(T(tmin:tmax),smooth(RAOstruct(k).Roll(tmin:tmax),10),'Color',cmap(k,:))
  
end
legend('0 deg', '90 deg')

title('Roll transfer functions')
xlabel('Frequency (Hz)')
ylabel('RAO (deg/m)')
 xlim([xmin xmax])
% ylim([0 5])

print(4,'-djpeg','Results/Platform/RAO_comparison/RollRAO');


%%

%Pitch
figure(5)
hold on
for k=1:2
    
   plot(T(tmin:tmax),smooth(RAOstruct(k).Pitch(tmin:tmax),10),'Color',cmap(k,:))
end
legend('0 deg', '90 deg')

title('Pitch transfer functions')
xlabel('Frequency (Hz)')
ylabel('RAO (deg/m)')
 xlim([xmin xmax])
% ylim([0 4])


print(5,'-djpeg','Results/Platform/RAO_comparison/PitchRAO');

%% Yaw
figure(6)
hold on
for k=1:2
    
    plot(T(tmin:tmax),smooth(RAOstruct(k).Yaw(tmin:tmax),10),'Color',cmap(k,:))
  
end
legend('0 deg', '90 deg')

title('Yaw transfer functions')
xlabel('Frequency (Hz)')
ylabel('RAO (deg/m)')
 xlim([xmin xmax])
% ylim([0 1])


print(6,'-djpeg','Results/Platform/RAO_comparison/YawRAO');

%% Wave 

figure(7)
hold on 
for k=1:2
    plot(fw,smooth(RAOstruct(k).Wave),'Color',cmap(k,:))
end
legend('0 deg','90 deg')

title('Wave spectrum')
xlabel('Frequency (Hz)')
ylabel('S(f) (m^2s)')

xlim([xmin xmax])

print(7,'-djpeg','Results/Platform/RAO_comparison/Wave_spectrum');
close all



